"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DevServerPlugin = void 0;
const utils_1 = require("@morjs/utils");
const net_1 = __importDefault(require("net"));
const os_1 = __importDefault(require("os"));
const DEFAULT_PORT = 8080;
const DEFAULT_HOST = '0.0.0.0';
/**
 * 尝试获取可用的端口号，最多重复获取 3 次
 * @param port - 端口号
 * @param host - ip 地址
 * @param retryTimes - 当前重试次数
 * @returns 可用的端口号
 */
async function checkPortInUseAndReturnAvaliable(port, host, retryTimes = 0) {
    const server = net_1.default.createServer();
    const availablePort = typeof port === 'string' ? parseInt(port) : port;
    const res = await new Promise(function (resolve) {
        server.once('error', function (err) {
            if (err.code === 'EADDRINUSE') {
                utils_1.logger.debug(`当前端口 ${availablePort} 已被占用`);
                resolve(0);
            }
        });
        server.once('listening', function () {
            server.close(function (err) {
                if (err) {
                    utils_1.logger.debug(`关闭 net server 失败，原因:`, err);
                    resolve(0);
                }
                else {
                    resolve(availablePort);
                }
            });
        });
        server.listen(availablePort, host);
    });
    // 重试 3 次
    if (!res && retryTimes < 3) {
        return checkPortInUseAndReturnAvaliable(availablePort + 1, host, retryTimes + 1);
    }
    else {
        return availablePort;
    }
}
/**
 * dev 服务器支持插件
 */
class DevServerPlugin {
    constructor(wrapper) {
        this.wrapper = wrapper;
        this.name = 'MorWebDevServerPlugin';
    }
    async apply(runner) {
        await this.setupDevServer(runner, this.wrapper, runner.userConfig);
    }
    /**
     * 配置 dev server
     */
    async setupDevServer(runner, wrapper, userConfig) {
        const { srcPaths = [], web } = userConfig;
        const host = runner.config.env.get('HOST') || DEFAULT_HOST;
        let port = runner.config.env.get('PORT') || DEFAULT_PORT;
        // 获取可用 port
        const availablePort = await checkPortInUseAndReturnAvaliable(port, host);
        if (String(availablePort) !== String(port)) {
            const answers = await (0, utils_1.prompts)([
                {
                    type: 'select',
                    name: 'suggestedPortAccepted',
                    message: `端口 ${port} 已被占用，使用 ${availablePort} 端口启动？`,
                    choices: [
                        { title: '是', value: true },
                        { title: '否', value: false }
                    ]
                }
            ], {
                onCancel() {
                    process.exit(0);
                }
            });
            if (answers.suggestedPortAccepted) {
                port = availablePort;
            }
        }
        // 启动 dev server
        wrapper.on('watch', () => {
            const devServerConfig = {
                port,
                host,
                historyApiFallback: true,
                static: srcPaths,
                hot: true,
                open: false,
                allowedHosts: 'all',
                devMiddleware: { stats: false },
                ...(web.devServer || {})
            };
            // 定制 logger
            wrapper.compiler.hooks.infrastructureLog.tap(this.name, function (name, type, args) {
                if (name === 'webpack-dev-server') {
                    const [msgOrError, ...restArgs] = args || [];
                    if (type === 'error' && msgOrError instanceof Error) {
                        const msg = msgOrError.message || msgOrError.name || '未知错误';
                        const error = msgOrError;
                        utils_1.logger.error(msg, { error });
                    }
                    else {
                        utils_1.logger.debug(msgOrError, ...restArgs);
                    }
                }
                return true;
            });
            let devServer = new utils_1.WebpackDevServer(devServerConfig, wrapper.compiler);
            // 输出服务链接
            let lastPrintTime = 0;
            wrapper.compiler.hooks.done.tapPromise(this.name, async () => {
                var _a, _b, _c;
                if (!devServer)
                    return;
                // 2 分钟最多打印一次
                if (Date.now() - lastPrintTime < 2 * 60 * 1000)
                    return;
                const ip = this.getIpAddress((_a = devServer === null || devServer === void 0 ? void 0 : devServer.options) === null || _a === void 0 ? void 0 : _a.host);
                const scheme = ((_c = (_b = devServer === null || devServer === void 0 ? void 0 : devServer.options) === null || _b === void 0 ? void 0 : _b.server) === null || _c === void 0 ? void 0 : _c.type) ===
                    'http'
                    ? 'http'
                    : 'https';
                const url = `${scheme}://${ip}:${devServerConfig.port}`;
                const qrcode = await (0, utils_1.generateQrcodeForTerminal)(url);
                utils_1.logger.info(`浏览器中打开链接: ${utils_1.COLORS.info(url)}\n或通过手机扫描下方二维码预览 👇\n\n` +
                    qrcode +
                    '\n');
                lastPrintTime = Date.now();
            });
            devServer.start();
            // runner 关闭时自动关闭 server
            runner.hooks.shutdown.tapPromise(this.name, async function () {
                if (devServer)
                    await devServer.stop();
                devServer = null;
            });
        });
    }
    /**
     * 获取本机 ip
     */
    getIpAddress(host = DEFAULT_HOST) {
        if (host !== DEFAULT_HOST)
            return host;
        const interfaces = os_1.default.networkInterfaces();
        for (const devName in interfaces) {
            const iFace = interfaces[devName];
            for (let i = 0; i < iFace.length; i++) {
                const alias = iFace[i];
                if (alias.family === 'IPv4' &&
                    alias.address !== '127.0.0.1' &&
                    !alias.internal) {
                    return alias.address;
                }
            }
        }
        return '127.0.0.1';
    }
}
exports.DevServerPlugin = DevServerPlugin;
//# sourceMappingURL=devServerPlugin.js.map